"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

import sys
import json
import yaml
import boto3
import botocore

from . import cli, utils, workplan, commands, output


def main():
    """ main function """

    # create a dict of globals which are accessed by different threads
    # this isn't strictly necessary but helps to highlight where we need thread safety
    utils.GLOBALS["stop_because_of_error"] = False
    utils.GLOBALS["thread_count"] = 0

    # process command line arguments...
    options = cli.check_args()

    # setup logging
    logger = utils.setup_logging(options)
    logger.debug("Got optoins: %s", options)

    # if a profile option was specified then set it up...
    if options.profile:
        try:
            logger.info("loading aws configuration profile: %s", options.profile)
            boto3.setup_default_session(profile_name=options.profile)
        except botocore.exceptions.BotoCoreError as bce:
            print("error: " + format(bce))
            sys.exit(1)

    # create boto3 clients...
    logger.debug("Creating AWS clients")
    try:
        org = boto3.client("organizations")
        sts = boto3.client("sts")
    except botocore.exceptions.BotoCoreError as bce:
        print("error: " + format(bce))
        sys.exit(1)

    # main program logic...
    workplan.examine_regions(logger, options)
    workplan.examine_accounts(logger, options, org)
    workplan.examine_command(logger, options)
    commands_list = workplan.build_work_plan(logger, options, sts)

    # make sure we have at least one command to run...
    if not commands_list:
        print("warning: no matching accounts/regions - nothing to do...")
        sys.exit(1)

    # don't bother using a thread pool if there is only one command...
    if not options.command:
        logger.debug("Calling run_command_unsafe() without threadpool as shell command")
        commands.run_command_unsafe(logger, options, commands_list[0])
        sys.exit(0)

    workplan.execute_work_plan(logger, options, commands_list)
    outputs = output.gather_command_outputs(logger, options, commands_list)


    logger.info("Nearly done, collating thread outputs")
    if options.format == "json":
        print(json.dumps(outputs, indent=4, sort_keys=True))
    elif options.format == "yaml":
        print(yaml.safe_dump(outputs, default_flow_style=False,
                             encoding="utf-8", allow_unicode=True, indent=4))
    else:
        for out in outputs:
            header = "{}@{} in {}:".format(out["role"], out["account"], out["region"])
            print("-" * len(header))
            print(header)
            print("-" * len(header))
            print("")
            print(out["output"])
            print("")
            print("")

if __name__ == '__main__':
    main()



"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

import os
import subprocess
import json
import copy
from . import utils


def run_command(logger, options, command_list):
    """ safe version for run_command_unsafe that takes care of locks """
    try:
        logger.debug("calling run_command_unsafe...")
        run_command_unsafe(logger, options, command_list)
    finally:
        utils.GLOBALS["thread_count"] = utils.GLOBALS["thread_count"] - 1
        logger.debug("commands still remaining: %s", utils.GLOBALS["thread_count"])
        if utils.GLOBALS["thread_count"] == 0:
            utils.GLOBALS["main_thread_lock"].release()
        logger.debug("thread is finished, releasing lock")
        utils.GLOBALS["thread_pool_lock"].release()


def run_command_unsafe(logger, options, command_list):
    """ run a command """

    if utils.GLOBALS["stop_because_of_error"]:
        return

    env = copy.deepcopy(os.environ)
    env.update(command_list["environment"])
    output = {}

    # check if this is a single command to run a SHELL...
    if not command_list["command"]:
        logger.debug("launching shell: %s", os.environ["SHELL"])
        command_list["command"] = os.environ["SHELL"]
        prompt = "[\\u({}:{})@\\h \\W]\\$ "
        env["PS1"] = prompt.format(command_list["account"], command_list["role"])
        subprocess.call(command_list["command"], env=env)
        return

    # copy some details from the command request to the command output...
    if isinstance(command_list["account"], dict):
        output["account"] = command_list["account"]["Id"]
        output["path"] = command_list["account"]["Path"]
    else:
        output["account"] = command_list["account"]

    output["role"] = command_list["role"]
    output["region"] = command_list["region"]
    output["command"] = " ".join(command_list["command"])

    # run the command and capture the output...
    try:
        output["output"] = subprocess.check_output(command_list["command"],
                                                   env=env, stderr=subprocess.STDOUT,
                                                   shell=False, universal_newlines=True)
        # try and parse the command output as JSON...
        try:
            output["output"] = json.loads(output["output"])
        except (ValueError, SyntaxError):
            pass

        command_list["output"] = output

    except subprocess.CalledProcessError as cpe:
        logger.info("Command returned non-zero exit code")
        output["error"] = {}
        output["error"]["message"] = format(cpe)
        output["output"] = cpe.output
        output["error"]["returncode"] = cpe.returncode
        command_list["output"] = output
        if options.stop_on_error:
            utils.GLOBALS["stop_because_of_error"] = True

    except OSError as ose:
        logger.info("Command failed to start")
        output["error"] = {}
        output["error"]["message"] = format(ose)
        command_list["output"] = output
        if options.stop_on_error:
            utils.GLOBALS["stop_because_of_error"] = True



"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

from . import utils


def get_child_ous(logger, org_client, org_unit):
    """ given an OU, find all the OUs within that OU... """
    logger.debug("Getting OUs for: %s", org_unit)
    result = [org_unit]

    # for this OU, get all the children...
    args = dict(ParentId=org_unit["Id"])
    children = utils.generic_paginator(logger, org_client.list_organizational_units_for_parent,
                                       "OrganizationalUnits", **args)

    # update child paths and then call ourselves recursively to find all children
    for child in children:
        child["Path"] = "{}/{}".format(org_unit["Path"], child["Name"]).replace("//", "/")
        result.extend(get_child_ous(logger, org_client, child))

    return result


def get_ou_from_path(logger, org_client, path):
    """ given a path, traverse Organizations OUs to locate the required OU... """
    logger.debug("Getting OU from path: %s", path)

    current_ou = org_client.list_roots()["Roots"][0]["Id"]
    if path == "/":
        return {"Id":current_ou, "Path":path}

    for dir_name in path.split("/")[1:]:
        logger.debug("Getting OU from path: %s, looking for: %s", path, dir_name)
        found = False
        args = dict(ParentId=current_ou)
        children = utils.generic_paginator(org_client.list_organizational_units_for_parent,
                                           "OrganizationalUnits", **args)

        for org_unit in children:
            if org_unit["Name"] == dir_name:
                current_ou = org_unit["Id"]
                found = True
                break

        if not found:
            raise ValueError("OU path not found")

    return {"Id":current_ou, "Path":path}


def get_accounts_for_ou(logger, options, org_client, path):
    """ given a path, get all the AWS accounts within that part of an Organization... """
    logger.debug("Getting accounts for OU: %s", path)
    org_unit = get_ou_from_path(logger, org_client, path)
    ous = []
    if options.no_recursive:
        ous.append(org_unit)
    else:
        ous.extend(get_child_ous(logger, org_client, org_unit))

    result = []
    for org_unit in ous:
        args = {"ParentId":org_unit["Id"]}
        accounts = utils.generic_paginator(logger, org_client.list_accounts_for_parent,
                                           "Accounts", **args)
        for acc in accounts:
            acc["Path"] = org_unit["Path"]
        result.extend(accounts)
    return result



"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""


def gather_command_outputs(logger, options, commands_list):
    """ gather the command outputs together """
    outputs = []
    for cmd in commands_list:

        # check if we have an output object that shouldn't be suppressed from --quiet...
        if "output" in cmd.keys():
            command_output = cmd["output"]["output"]
            oo_single_key = False
            oo_single_empty_key = False
            if isinstance(command_output, dict):
                oo_key_count = len(command_output.keys())
                oo_single_key = oo_key_count == 1
                if oo_single_key:
                    oo_key_values = len(command_output[command_output.keys()[0]])
                    oo_single_empty_key = oo_key_values == 0

            # check for literally no output...
            if options.quiet and (command_output is None or command_output):
                logger.debug("Command output is actually empty, skipping: %s", cmd["output"])

            # check if output contains a single empty object...
            elif options.quiet and oo_single_empty_key:
                logger.debug("Command output is effectively empty, skipping: %s", cmd["output"])

            else:
                outputs.append(cmd["output"])
    return outputs



"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

import sys
import os
import threading
import boto3
import botocore

from . import regions, utils, organizations, commands

def examine_regions(logger, options):
    """ for each region provided, use it as a regex to search for regions... """
    logger.debug("Getting list of regions")
    regions_list = regions.get_regions_list(logger)
    if options.regions:
        matched_regions = map(lambda x: regions.get_regions_from_regex(logger, x, regions_list),
                              options.regions)
        options.regions = sorted(set(utils.flatten_list(matched_regions)))
        if not options.regions:
            print("error: no matching regions found")
            sys.exit(1)
        logger.info("Set regions: %s", ", ".join(options.regions))

    # if we don't have any regions set, then try and get it from the current session...
    if not options.regions:
        logger.debug("No regions specified on command line, guessing...")
        options.regions = [boto3.session.Session().region_name]
        if options.regions == [None]:
            options.regions = ["<default>"]
            logger.info("No region set, using default")
        else:
            logger.info("Set regions: %s", ", ".join(options.regions))


def examine_accounts(logger, options, org_client):
    """ if we don't have any accounts set, then try and get guess our current account_id... """
    logger.debug("Getting list of accounts")
    if not options.accounts:
        logger.debug("No accounts specified on command line, guessing...")
        try:
            # if we have any kind of AWS credentials set then this should work...
            account_id = boto3.client("sts").get_caller_identity()["Account"]
            options.accounts = [account_id]
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError):
            print("error: unable to work out existing account ID, "
                  "please check your AWS credentials, or specify it with the '-a' option")
            sys.exit(1)

    logger.info("Set accounts: %s", ", ".join(options.accounts))

    # if we were given OUs then convert them into a list of accounts
    logger.debug("Checking if we need to traverse an Organization")
    if options.ous:
        logger.debug("Getting list of accounts from OUs")
        mapped_list = map(lambda path: organizations.get_accounts_for_ou(logger, options,
                                                                         org_client, path),
                          options.ous)
        options.accounts = utils.flatten_list(mapped_list)
        if options.no_master:
            master = org_client.describe_organization()["Organization"]["MasterAccountId"]
            logger.debug("Removing the master account (%s) from the list of accounts", master)
            options.accounts = filter(lambda x: x["Id"] != master, options.accounts)

    logger.info("Set accounts: %s", options.accounts)


def examine_command(logger, options):
    """ look at the command and guess if it is an AWS CLI built in command """
    if not options.no_cli_guess and options.command:
        try:
            import awscli.clidriver
            logger.debug("Guessing if the supplied command is an AWS CLI command..")
            cli = awscli.clidriver.CLIDriver()
            cli_help = cli.create_help_command()
            aws_cli_commands = map(lambda x: x, cli_help.command_table)

            if options.command[0] in aws_cli_commands:
                options.commands.insert(0, "aws")
                logger.debug("Assuming command is an AWS CLI, new command is: %s", options.command)
        except ImportError:
            logger.debug("awscli module not found or failed to load")


def build_work_plan(logger, options, sts_client):
    """ create a big list of commands we need to run... """
    logger.info("Starting analysis on work plan")
    commands_list = []

    # iterate over accounts and regions...
    for account in options.accounts:

        logger.debug("Looking at account: %s", account)
        account_id = account["Id"] if isinstance(account, dict) else account

        # work out if we need to call STS to assume a new role...
        if options.role:

            # call STS to get credentials for the account and role...
            try:
                arn = "arn:aws:iam::{}:role/{}".format(account_id, options.role)
                logger.debug("Calling STS to get temporary credentials for: %s", arn)
                assumed_role = sts_client.assume_role(
                    RoleArn=arn,
                    RoleSessionName="{}@{}".format(os.environ["USER"], os.environ["HOSTNAME"]),
                    ExternalId="{}@{}".format(os.environ["USER"], os.environ["HOSTNAME"])
                )

            except botocore.exceptions.BotoCoreError as bce:
                print("error switching role ({}@{}): {}".format(options.role,
                                                                account_id, bce.args))
                sys.exit(1)

        for region in options.regions:
            logger.debug("Looking at region: %s", region)
            cmd = {}
            cmd["command"] = options.command
            cmd["environment"] = {}
            cmd["role"] = options.role
            cmd["account_id"] = account_id
            cmd["account"] = account
            cmd["region"] = region
            if region != "<default>":
                cmd["environment"]["AWS_DEFAULT_REGION"] = region
            if options.role:
                cmd["environment"]["AWS_ACCESS_KEY_ID"] = assumed_role["Credentials"]["AccessKeyId"]
            logger.debug("Adding command to work plan: %s", cmd)
            cmd["options"] = options

            if options.role:
                env = cmd["environment"]
                env["AWS_SECRET_ACCESS_KEY"] = assumed_role["Credentials"]["SecretAccessKey"]
                env["AWS_SESSION_TOKEN"] = assumed_role["Credentials"]["SessionToken"]
            commands_list.append(cmd)

    return commands_list


def execute_work_plan(logger, options, commands_list):
    """ run through commands_list and run various commands in the thread pool """
    logger.info("Executing work plan across a thread pool of size: %s", options.threads)
    utils.GLOBALS["main_thread_lock"] = threading.Lock()
    utils.GLOBALS["thread_pool_lock"] = threading.BoundedSemaphore(options.threads)
    utils.GLOBALS["thread_count"] = len(commands_list)
    logger.debug("Locks created, task list size = %s", utils.GLOBALS["thread_count"])

    # obtain the main thread lock...
    logger.debug("Acquiring main thread lock")
    utils.GLOBALS["main_thread_lock"].acquire()

    for cmd in commands_list:
        logger.debug("waiting for next thread to be available")
        utils.GLOBALS["thread_pool_lock"].acquire()
        logger.debug("thread is available, starting thread")
        threading.Thread(target=commands.run_command, args=(logger, options, cmd, )).start()

    # block on the main thread lock being released...
    logger.debug("Blocking main thread, waiting on commands to finish")
    utils.GLOBALS["main_thread_lock"].acquire()
    logger.debug("Main thread lock released, working on output")



"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

import itertools
import logging

GLOBALS = {}

def flatten_list(the_list):
    """ take a list of lists and flatten it to just a list """
    return [] if the_list is None else list(itertools.chain.from_iterable(the_list))


def split_list(the_list, splitter):
    """ take a list and split each item and put the split items back into the main list"""
    return [] if the_list is None else flatten_list(map(lambda x: x.split(splitter), the_list))


def generic_paginator(logger, paged_function, result_object, **kwargs):
    """ call an API until there are no more results """
    logger.debug("in generic_paginator, for: %s", paged_function)
    for key, value in kwargs.items():
        logger.debug("                   params: %s=%s", key, value)
    full_results = []
    next_token = ""
    while next_token is not None:
        if next_token != "":
            kwargs.update({"NextToken":next_token})
        result = paged_function(**kwargs)
        full_results.extend(result[result_object])
        next_token = result.get("NextToken", None)
        logger.debug("               next_token: %s", next_token)
    return full_results


def setup_logging(options):
    """ set up logging... """
    logger = logging.getLogger("aws-with")
    logger.setLevel(logging.ERROR)
    stream_handler = logging.StreamHandler()
    formatter = logging.Formatter('%(asctime)s %(levelname)s(%(threadName)s): %(message)s')
    stream_handler.setFormatter(formatter)
    logger.addHandler(stream_handler)
    if options.verbosity >= 1:
        logger.setLevel(logging.INFO)
    if options.verbosity >= 2:
        logger.setLevel(logging.DEBUG)
    logger.debug("Logger set up")
    return logger



""" aws_with module definition """

from . import utils
from . import regions
from . import organizations
from . import commands
from . import workplan
from . import output
from . import cli
from . import main

VERSION = "0.9.1"
BUILD_TIME = "2017-08-27 11:14:00"
BUILD_COMMIT_HASH = "0012300400g0hf020304f34f"



"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

import sys
import argparse
from . import utils


def create_args_parser():
    """ process command line arguments and perform sanity checks """
    parser = argparse.ArgumentParser(description="Run the same command across a number of "
                                                 "AWS accounts and/or regions.",
                                     usage="%(prog)s [options] [COMMAND]",
                                     formatter_class=argparse.RawDescriptionHelpFormatter,
                                     epilog="""
NOTE:  -R, -o and -a can take a comma-separated list or the option may be given multiple times.

EXAMPLES:

    aws-with --role=admins-role
    Assume the IAM role 'admins-role' within the current AWS account and then run a shell.

    aws-with --role=admins-role --accounts=123456789012
    Assume the IAM role 'admins-role' in the account 123456789012 and then run a shell.

    aws-with --regions='us-*' ec2 describe-instances
    List all EC2 instances across all US regions.

    aws-with --ous=/Technology/Development s3 ls
    List all S3 buckets owned by any account under the Organizations OU 'Development'.
    The 'Development' OU is itself under the 'Technology' OU off the Root OU.

    aws-with -o /Production,/Staging/Final -R '*' \\
        cloudformation update-stack --stack-name 'security-checks' ...
    Update a stack across all accounts under the 'Production' Organizations OU
    or under the /Staging/Final OU and across all regions.
""")

    parser.add_argument("-R", "--regions",
                        dest="regions", action="append",
                        help="Run the command for all regions that match PATTERNS")

    parser.add_argument("-r", "--role",
                        dest="role", action="store",
                        help="Use STS assumeRole to take on a different IAM role. If not "
                             "specified then ROLE defaults to OrganizationAccountAccessRole")

    parser.add_argument("-o", "--ous",
                        dest="ous", action="append",
                        help="Run the command for all child accounts under "
                             "the Organizations OU PATHS")

    parser.add_argument("-a", "--accounts",
                        dest="accounts", action="append",
                        help="Run the command for all the listed ACCOUNTS")

    parser.add_argument("-x", "--no-recursive",
                        dest="no_recursive", action="store_true",
                        help="When scanning Organizations for accounts, don't look recursively")

    parser.add_argument("-t", "--threads",
                        dest="threads", action="store", default=2, type=int,
                        help="Set the number of threads to use when running commands (default: 10)")

    parser.add_argument("-f", "--output",
                        dest="format", action="store", default="json",
                        type=str, choices=["json", "yaml", "text"],
                        help="Set the output format to use")

    parser.add_argument("-q", "--quiet",
                        dest="quiet", action="store_true",
                        help="Suppress output if a command is successful but has no output")

    parser.add_argument("-e", "--stop-on-error",
                        dest="stop_on_error", action="store_true",
                        help="Stop running commands if one throws an error")

    parser.add_argument("-v", "--verbose",
                        dest="verbosity", action="count",
                        help="Output debug messages, increase messages with -v -v")

    parser.add_argument("-g", "--no-cli-guess",
                        dest="no_cli_guess", action="store_true",
                        help="Do not attempt to guess if the command is an AWS CLI command")

    parser.add_argument("-m", "--no-master",
                        dest="no_master", action="store_true",
                        help="Do not include the Organizations master account in searches")

    parser.add_argument("-p", "--profile",
                        dest="profile", action="store",
                        help="Use the saved AWS credentials/profile called PROFILE")

    parser.add_argument("command", nargs=argparse.REMAINDER, metavar="COMMAND",
                        help="This is the command that should be run across regions/accounts/OUs."
                             "  COMMAND is mandatory when the -R or -o options are used, "
                             "otherwise it is optional and a shell will be run if it is omitted.")

    return parser

def error(message):
    """ display an error related to command line arguments and quit """
    print(message + "\nhint: try -h for help")
    sys.exit(1)

def args_basic_checks(parsed_options):
    """ perform basic sanity checks on the command line arguments """
    check_none = [parsed_options.role, parsed_options.regions,
                  parsed_options.ous, parsed_options.accounts]
    if check_none == [None]*len(check_none):
        error("error: you must specify at least one of -r, -R, -o or -a")

    if parsed_options.accounts and parsed_options.ous:
        error("error: you cannot specify both -a and -o")

    if parsed_options.regions and parsed_options.command == 0:
        error("error: if you specify -R or -o then you must supply a command to run")

    if parsed_options.ous and parsed_options.command == 0:
        error("error: if you specify -R or -o then you must supply a command to run")

    if len(parsed_options.accounts) > 1 and parsed_options.command:
        error("error: if you specify multiple accounts with -a then "
              "you must supply a command to run")

def check_args():
    """ process command line arguments """
    parser = create_args_parser()
    parsed_options = parser.parse_args()
    args_basic_checks(parsed_options)

    # if -a or -o are specified the -r has a default value so set it...
    if parsed_options.accounts is not None or parsed_options.ous is not None:
        if parsed_options.role is None:
            parsed_options.role = "OrganizationAccountAccessRole"

    # expand out parsed_options that can take a list of values...
    parsed_options.ous = sorted(set(utils.split_list(parsed_options.ous, ",")))
    parsed_options.accounts = sorted(set(utils.split_list(parsed_options.accounts, ",")))
    parsed_options.regions = sorted(set(utils.split_list(parsed_options.regions, ",")))
    return parsed_options



from setuptools import setup
import aws_with

setup(
   name='aws_with',
   version=aws_with.VERSION,
   description='Command line utility to help manage AWS accounts, regions and organizations',
   license="APACHE2",
   long_description=open("README.rst").read(),
   author='Mike Evans',
   author_email='miev@amazon.com',
   packages=['aws_with'],
   install_requires=["boto3", "PyYAML"],
   platforms='any',
   data_files = [("", ["LICENSE.txt"])],
   entry_points={'console_scripts': ['aws_with = aws_with.main:main']},
   classifiers=['Development Status :: 4 - Beta',
                'Environment :: Console',
                'Intended Audience :: Developers',
                'Intended Audience :: End Users/Desktop',
                'Intended Audience :: Information Technology',
                'Intended Audience :: System Administrators',
                'License :: OSI Approved :: Apache Software License',
                'Operating System :: POSIX',
                'Operating System :: Microsoft :: Windows',
                'Operating System :: MacOS :: MacOS X',
                'Topic :: System :: Systems Administration',
                'Topic :: Utilities',
                'Programming Language :: Python',
                'Programming Language :: Python :: 2.6',
                'Programming Language :: Python :: 2.7',
                'Programming Language :: Python :: 3.4',
                'Programming Language :: Python :: 3.5',
                'Programming Language :: Python :: 3.6',
                'Programming Language :: Python :: 3.7'],
)

